﻿using Caliburn.Micro;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Threading;

namespace TelerikDemo
{
    public class FocusHelper
    {
        private static bool isFocusedByClick;

        private static void OnEnsureFocusChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = d as Control;
            if (!(bool)e.NewValue && GetIsRestoreFocusEnabled(control) && isFocusedByClick)
            {
                control.Dispatcher.BeginInvoke(new System.Action(() =>
                {
                    Message.SetAttach(control, string.Empty);
                    control.Focus();
                    Message.SetAttach(control, "[Event GotFocus] = [BusyTrigger]");
                    isFocusedByClick = false;
                }));
            }
        }

        public static bool GetEnsureFocus(DependencyObject obj)
        {
            return (bool)obj.GetValue(EnsureFocusProperty);
        }

        public static void SetEnsureFocus(DependencyObject obj, bool value)
        {
            obj.SetValue(EnsureFocusProperty, value);
        }

        public static readonly DependencyProperty EnsureFocusProperty =
            DependencyProperty.RegisterAttached(
            "EnsureFocus",
            typeof(bool),
            typeof(FocusHelper),
            new PropertyMetadata(OnEnsureFocusChanged));



        public static bool GetIsRestoreFocusEnabled(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsRestoreFocusEnabledProperty);
        }

        public static void SetIsRestoreFocusEnabled(DependencyObject obj, bool value)
        {
            obj.SetValue(IsRestoreFocusEnabledProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsRestoreFocusEnabled.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsRestoreFocusEnabledProperty =
            DependencyProperty.RegisterAttached("IsRestoreFocusEnabled", typeof(bool), typeof(FocusHelper), new PropertyMetadata(false, OnIsRestoreFocusEnabledChanged));

        private static void OnIsRestoreFocusEnabledChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = d as Control;
            if (control != null && e.NewValue != e.OldValue)
            {
                if ((bool)e.NewValue)
                {
                    control.PreviewMouseLeftButtonDown += Control_PreviewMouseLeftButtonDown;
                }
                else
                {
                    control.PreviewMouseLeftButtonDown -= Control_PreviewMouseLeftButtonDown;
                }
            }
        }

        private static void Control_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            isFocusedByClick = true;
        }
    }
}
